@@ -1,6 +1,6 @@
 <?php
 /**
- * @version   v5.20.9  21-Dec-2016
+ * @version   v5.20.16  12-Jan-2020
  * @copyright (c) 2000-2013 John Lim (jlim#natsoft.com). All rights reserved.
  * @copyright (c) 2014      Damien Regad, Mark Newnham and the ADOdb community
  * Released under both BSD license and Lesser GPL library license.
@@ -111,7 +111,7 @@ function adodb_error_pg($errormsg)
 			'could not serialize access due to'   => DB_ERROR_SERIALIZATION_FAILURE
 		);
 	reset($error_regexps);
-	while (list($regexp,$code) = each($error_regexps)) {
+	foreach ($error_regexps as $regexp => $code) {
 		if (preg_match("/$regexp/mi", $errormsg)) {
 			return $code;
 		}
